import torch
import torch.nn as nn
from TripleModels import TripleModels
from torch.autograd import Variable
import numpy as np


class Network(nn.Module):
    def __init__(self,index ):
        super(Network, self).__init__()        
        if index==61:
            self.Models = TripleModels()
            print( 'Using TripleModels' )
        self.softmax = nn.LogSoftmax() 
        self.loss_softmax = nn.CrossEntropyLoss().cuda()
    @property
    def loss(self):
        return self.loss_label
    
    def forward(self,  im_data,dem_data,slope_data,curvation_data, img_data,wetidx_data,flowacc_data,index,gt_data=None):       

        im_data        = np_to_variable(im_data, is_cuda=True, is_training=self.training)                
        dem_data       = np_to_variable(dem_data, is_cuda=True, is_training=self.training)                
        slope_data     = np_to_variable(slope_data, is_cuda=True, is_training=self.training)   
        img_data       = np_to_variable(img_data, is_cuda=True, is_training=self.training) 

        curvation_data  = [] 
        wetidx_data     = [] 
        flowacc_data    = [] 
               
        featuremap = {}
        if index==61:
            pre_label = self.Models(img_data,dem_data,slope_data,0)
       
  
        if self.training:      
            b = torch.LongTensor(gt_data.tolist())
            gt_data = Variable( b )
            gt_data = gt_data.cuda()
            gt_data = gt_data.view(-1)            
            self.loss_label = self.build_loss(pre_label, gt_data)

        if self.training:      
            return pre_label
        else:
            return pre_label,featuremap

        #return pre_label
    def build_loss(self, pre_label, gt_label):
        loss = self.loss_softmax(pre_label, gt_label)
        return loss

class Conv2d(nn.Module):
    def __init__(self, in_channels, out_channels, kernel_size, stride=1, relu=True, same_padding=False, bn=False):
        super(Conv2d, self).__init__()
        padding = int((kernel_size - 1) / 2) if same_padding else 0
        self.conv = nn.Conv2d(in_channels, out_channels, kernel_size, stride, padding=padding)
        self.bn = nn.BatchNorm2d(out_channels, eps=0.001, momentum=0, affine=True) if bn else None
        self.relu = nn.ReLU(inplace=True) if relu else None

    def forward(self, x):
        x = self.conv(x)
        if self.bn is not None:
            x = self.bn(x)
        if self.relu is not None:
            x = self.relu(x)
        return x

def np_to_variable(x, is_cuda=True, is_training=False, dtype=torch.FloatTensor):
    
    if is_training:
        v = Variable(torch.from_numpy(x).type(dtype))
    else:
        v = Variable(torch.from_numpy(x).type(dtype), requires_grad = False, volatile = True)

    v = v.cuda()
    return v


def set_trainable(model, requires_grad):
    for param in model.parameters():
        param.requires_grad = requires_grad

